library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity gc_ide is
	Port(
		ide_adr : out std_logic_vector(4 downto 0);  -- /CS0, /CS1, A2..A0
		ide_dat : inout std_logic_vector(15 downto 0); -- D15..D0
		ide_rd : out std_logic; -- /IOR
		ide_wr : out std_logic; -- /IOW
		--ide_reset : out std_logic;
    	
		--sparep : in std_logic_vector(14 downto 0); 
	   
	--	led_inp : in std_logic;
	--	led_out : out std_logic;
		
	-- exi
		exi_in  : in std_logic;
		exi_out : out std_logic;
		exi_clk : in std_logic;
		exi_cs  : in std_logic	);
  
end gc_ide;

architecture behavioral of gc_ide is

--********************************************************
subtype bit8 is integer range 0 to 255;

signal ide_adr_host_sig: std_logic_vector(4 downto 0);
signal ide_host_rd: std_logic;
signal ide_host_wr: std_logic;
signal ide_host_data: std_logic_vector(15 downto 0);

signal exi_count: bit8;
signal exi_data0:  std_logic_vector(15 downto 0);
signal exi_data1:  std_logic_vector(15 downto 0);
signal exi_out_latch: std_logic;

signal s_exi_out : std_logic;
signal s_exi_in : std_logic;

-- exi protocol
signal exi_address: std_logic_vector(4 downto 0); -- cs0, cs1, a2..a0 (5 bits)
signal do_read: std_logic;
signal do_write: std_logic;

--signal rdmult_cnt: bit8;
signal do_8bit: std_logic;
signal rdmult_cnt: std_logic_vector(15 downto 0);
signal rdmult: std_logic := '0';
signal inirdmult: std_logic := '0';

begin

--LEDIndicator:
--led_out <= NOT led_inp;
  

--******************************************************************

Switch_Global:
process (  ide_adr_host_sig, ide_host_data, ide_host_rd, ide_host_wr )
begin
     	--ide_reset <= '1';
		
		ide_adr <= ide_adr_host_sig;  -- saca direcciones
		
		if ide_host_wr = '0' then
		    ide_dat <= ide_host_data; -- escritura: saca datos
		else
          ide_dat <= (others => 'Z');		
		end if;
		
		ide_rd <= ide_host_rd;
		ide_wr <= ide_host_wr;
		
end process;

HOSTWrRdIDE:
process( exi_cs, exi_clk )
	-- 31..24 command phase: r/w, 8/16, nu, cs1,cs0,a2,a1,a0
	-- 23..16 dummy in read/data lsb in write  
	-- 15..8  data lsb in read/data msb in write16, or dummy in write8
	--  7..0  dummy in write16/data msb in read  

begin

	if (exi_cs = '1') then     -- when chip select disabled, just Z

		exi_count <= 31;
		exi_out <= 'Z';
		ide_adr_host_sig <= (others => '0');
		ide_dat <= (others => 'Z');
		ide_host_rd <= '1';
		ide_host_wr <= '1';
		do_read <= '0';
		do_write<= '0';
		do_8bit <= '0';
		inirdmult <= '0';

	else

	    exi_out <= exi_out_latch;
	
	    if (exi_clk='1' and exi_clk'event) then       
	
			case exi_count is
				when 31 =>
					   -- w/r command
						if (exi_in='0') or (rdmult='1') then
							do_read <= '1';
						else
							do_write <= '1'; 
						end if; 

						-- in case of read multiple data1 is valid... 
						-- exicount 31 (data1 bit 6)
						exi_out_latch <= exi_data1(6);						
				
				when 30 =>	
						-- 16/8 command   
						if (exi_in='0') and (do_write='1') then
							do_8bit <= '1';
						end if;  
						
						-- in case of read multiple data1 is valid...
					   -- exicount 30 (data1 bit 5)
						exi_out_latch <= exi_data1(5);

				when 29 =>	
						-- rd multiple command  
						if (exi_in='1') and (do_read='1') and (rdmult='0') then
							inirdmult <= '1';   -- first rdmult cycle
							do_8bit <= '0';
						end if;  
					
					    -- exicount 29 (data1 bit 4)
					    exi_out_latch <= exi_data1(4);
					
				when 28 downto 24 =>	
					-- address   
					if (inirdmult='1') or (rdmult='1') then
      					exi_address(4 downto 0)<= "10000";  -- ide select data register
					else
						exi_address <= exi_address(3 downto 0) & exi_in;					    
					end if;
					
					-- read_multiple 
				   if exi_count=24 then
						   -- exicount 24 (data1 bit 15)
						   exi_out_latch <= exi_data1(15);
					else
						   -- exicount 28...25  (data1 bits 3...0)
						   exi_out_latch <= exi_data1(exi_count-25);	
					end if;			
					
				when 23 downto 16 =>  
				    	if do_read='1' then
						
							if inirdmult='0' then
								-- if read generate RD pulse and get ide data0			
								if exi_count >= 20 then
									ide_host_rd <= '0';	 	-- init rd pulse
								else 
									exi_data0 <= ide_dat; 	-- get data in bus
									ide_host_rd <='1'; 		-- end pulse						            					            						            
								end if;
							else
                        -- receive dwords to read (only in first rdmult cycle)
								rdmult_cnt(exi_count-16) <= exi_in; -- 7..0 / LSB			
							end if;
						
							--exicount 16 (data0 bit 7)
							if exi_count = 16 then
								exi_out_latch <= exi_data0(7);
							else
							--exicount 23..17 (data1 bits 14..8)
								exi_out_latch <= exi_data1(exi_count-9);
							end if;
							
						-- if write, receive data from exi  
						else 
						-- receive first byte (write op)
							exi_data0(exi_count-16) <= exi_in; -- 7..0 / LSB			
						end if;   				
							
				when 15 downto 8 => 
						-- send/rec data  
						if do_read='1' then  
							-- transmit first readed byte (read op)
							if exi_count=8 then
							 exi_out_latch <= exi_data0(15);  -- 15 							
							else 
							 exi_out_latch <= exi_data0(exi_count-9);  -- 6..0 / LSB	
							end if;
							
							if inirdmult='1' then
								-- receive sectors to read (only in first rdmult cycle)
								rdmult_cnt(exi_count) <= exi_in; -- 15..8 / MSB			
							end if;	
							
						else
							-- receive last byte from exi (write op)
							exi_data0(exi_count) <= exi_in; -- 15..8 / MSB			
						end if;
						
				when 7 downto 0 =>
				   if do_write='1' then
						-- if write put data in bus and generate WR pulse 			
							if exi_count >= 4 then
								ide_host_wr <= '0';	 	-- init wr pulse
							   ide_host_data <= exi_data0;	-- put data to write in ide bus
							else 
						      ide_host_wr <='1'; 		-- end wr pulse
							end if;
						-- if read transmit last data  
					else
					   -- if read transmit last data0 and generate rd1 pulse, get data1 
						
					      if (rdmult='1') or (inirdmult='1') then
						    -- if read generate RD1 pulse and get ide data1			
							   if exi_count >= 4 then
								   ide_host_rd <= '0';	 	-- init rd1 pulse
								else 
									exi_data1 <= ide_dat; 	-- get data1 in bus
									ide_host_rd <='1'; 		-- end pulse rd1     					            						            
								end if;
							end if;	

						-- transmit  
						if exi_count=0 then
							-- exicount 0 (data1 bit 7)
							exi_out_latch <= exi_data1(7);
						else
							-- exicount 7..1 (data0 bits 14..8) 
							exi_out_latch <= exi_data0(exi_count+7); -- 14..8 / MSB
						end if;
							
					end if;   
				when others => NULL;
			end case;
			
			exi_count <= exi_count - 1;
			
			if (exi_count=15) and (do_8bit='1') then 
				exi_count <= 7; -- in case of 8 bit (write), skip 
			end if;	
			
			if inirdmult='1' then
				--rdmult_cnt <= 127; -- dwordcnt; -- set counter  (number of 32 bit words to read, one hdd sector=128 dwords)
				rdmult <= '1'; -- set flag
			else 
			
				if exi_count=0 then
					 rdmult_cnt <= rdmult_cnt-'1'; -- decrement dword count	( 32bits)

				    if rdmult_cnt=x"0" then 
						rdmult <= '0'; --finish rdmult operation
					 end if;
					 
				end if; 
				
			end if;
			
         --exi_out <= exi_out_latch;

        end if;

  	   ide_adr_host_sig <= exi_address(4 downto 0);
	   --exi_out <= exi_out_latch;

    end if;
	
end process;

end behavioral;

